@@ -10,7 +10,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #ifdef _WIN32
-#include <Windows.h>
+#include <windows.h>
 #include <direct.h>
 #include <io.h>
 #else
@@ -128,7 +128,7 @@ int main(int argc, char* argv[])
 	{
 	    QTC::TC("examples", "pdf-mod-info -key");
 	    cur_key = argv[i];
-	    if (! ((cur_key.length() > 0) && (cur_key[0] == '/')))
+	    if (! ((cur_key.length() > 0) && (cur_key.at(0) == '/')))
 	    {
 		cur_key = "/" + cur_key;
 	    }
@@ -209,7 +209,7 @@ int main(int argc, char* argv[])
 	QPDFWriter w(file, fl_tmp.c_str());
 	w.setStreamDataMode(qpdf_s_preserve);
 	w.setLinearization(true);
-	w.setStaticID(static_id);
+	w.setStaticID(static_id); // for testing only
 	w.write();
     }
     catch (std::exception& e)
