@@ -62,7 +62,7 @@ int main(int argc, char* argv[])
 	usage();
     }
     char const* filename = argv[1];
-    int pageno = atoi(argv[2]);
+    int pageno = QUtil::string_to_int(argv[2]);
 
     try
     {
@@ -75,9 +75,8 @@ int main(int argc, char* argv[])
         }
 
         QPDFObjectHandle page = pages.at(pageno-1);
-        QPDFObjectHandle contents = page.getKey("/Contents");
         ParserCallbacks cb;
-        QPDFObjectHandle::parseContentStream(contents, &cb);
+        page.parsePageContents(&cb);
     }
     catch (std::exception& e)
     {
