@@ -1,9 +1,23 @@
-/* Copyright (c) 2005-2014 Jay Berkenbilt
+/* Copyright (c) 2005-2018 Jay Berkenbilt
  *
- * This file is part of qpdf.  This software may be distributed under
- * the terms of version 2 of the Artistic License which may be found
- * in the source distribution.  It is provided "as is" without express
- * or implied warranty.
+ * This file is part of qpdf.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ *
+ * Versions of qpdf prior to version 7 were released under the terms
+ * of version 2.0 of the Artistic License. At your option, you may
+ * continue to consider qpdf to be licensed under those terms. Please
+ * see the manual for additional information.
  */
 
 #ifndef __QPDF_C_H__
@@ -166,6 +180,14 @@ extern "C" {
     QPDF_DLL
     void qpdf_set_suppress_warnings(qpdf_data qpdf, QPDF_BOOL value);
 
+    /* CHECK FUNCTIONS */
+
+    /* Attempt to read the entire PDF file to see if there are any
+     * errors qpdf can detect.
+     */
+    QPDF_DLL
+    QPDF_ERROR_CODE qpdf_check_pdf(qpdf_data qpdf);
+
     /* READ FUNCTIONS */
 
     /* READ PARAMETER FUNCTIONS -- must be called before qpdf_read */
@@ -318,14 +340,32 @@ extern "C" {
     void qpdf_set_stream_data_mode(qpdf_data qpdf,
 				   enum qpdf_stream_data_e mode);
 
+    QPDF_DLL
+    void qpdf_set_compress_streams(qpdf_data qpdf, QPDF_BOOL value);
+
+
+    QPDF_DLL
+    void qpdf_set_decode_level(qpdf_data qpdf,
+                               enum qpdf_stream_decode_level_e level);
+
+    QPDF_DLL
+    void qpdf_set_preserve_unreferenced_objects(
+        qpdf_data qpdf, QPDF_BOOL value);
+
+    QPDF_DLL
+    void qpdf_set_newline_before_endstream(qpdf_data qpdf, QPDF_BOOL value);
+
     QPDF_DLL
     void qpdf_set_content_normalization(qpdf_data qpdf, QPDF_BOOL value);
 
     QPDF_DLL
     void qpdf_set_qdf_mode(qpdf_data qpdf, QPDF_BOOL value);
 
+    QPDF_DLL
+    void qpdf_set_deterministic_ID(qpdf_data qpdf, QPDF_BOOL value);
+
     /* Never use qpdf_set_static_ID except in test suites to suppress
-     * generation of a random /ID.
+     * generation of a random /ID.  See also qpdf_set_deterministic_ID.
      */
     QPDF_DLL
     void qpdf_set_static_ID(qpdf_data qpdf, QPDF_BOOL value);
