@@ -356,6 +356,12 @@ QPDFWriter::setLinearization(bool val)
     }
 }
 
+void
+QPDFWriter::setLinearizationPass1Filename(std::string const& filename)
+{
+    this->m->lin_pass1_filename = filename;
+}
+
 void
 QPDFWriter::setPCLm(bool val)
 {
@@ -1246,8 +1252,28 @@ QPDFWriter::writeTrailer(trailer_e which, int size, bool xref_stream,
     writeString(" /ID [");
     if (linearization_pass == 1)
     {
-        writeString("<00000000000000000000000000000000>"
-                    "<00000000000000000000000000000000>");
+	std::string original_id1 = getOriginalID1();
+        if (original_id1.empty())
+        {
+            writeString("<00000000000000000000000000000000>");
+        }
+        else
+        {
+            // Write a string of zeroes equal in length to the
+            // representation of the original ID. While writing the
+            // original ID would have the same number of bytes, it
+            // would cause a change to the deterministic ID generated
+            // by older versions of the software that hard-coded the
+            // length of the ID to 16 bytes.
+            writeString("<");
+            size_t len = QPDF_String(original_id1).unparse(true).length() - 2;
+            for (size_t i = 0; i < len; ++i)
+            {
+                writeString("0");
+            }
+            writeString(">");
+        }
+        writeString("<00000000000000000000000000000000>");
     }
     else
     {
@@ -1565,7 +1591,8 @@ QPDFWriter::unparseObject(QPDFObjectHandle object, int level,
 	{
 	    is_metadata = true;
 	}
-	bool filter = (this->m->compress_streams ||
+	bool filter = (object.isDataModified() ||
+                       this->m->compress_streams ||
                        this->m->stream_decode_level);
 	if (this->m->compress_streams)
 	{
@@ -1576,7 +1603,8 @@ QPDFWriter::unparseObject(QPDFObjectHandle object, int level,
 	    // compressed with a lossy compression scheme, but we
 	    // don't support any of those right now.
 	    QPDFObjectHandle filter_obj = stream_dict.getKey("/Filter");
-	    if (filter_obj.isName() &&
+            if ((! object.isDataModified()) &&
+                filter_obj.isName() &&
 		((filter_obj.getName() == "/FlateDecode") ||
 		 (filter_obj.getName() == "/Fl")))
 	    {
@@ -1593,7 +1621,7 @@ QPDFWriter::unparseObject(QPDFObjectHandle object, int level,
 	    QTC::TC("qpdf", "QPDFWriter not compressing metadata");
 	    filter = true;
 	    compress = false;
-           uncompress = true;
+            uncompress = true;
 	}
 	else if (this->m->normalize_content &&
                  this->m->normalized_streams.count(old_og))
@@ -1952,6 +1980,20 @@ QPDFWriter::writeObject(QPDFObjectHandle object, int object_stream_index)
     }
 }
 
+std::string
+QPDFWriter::getOriginalID1()
+{
+    QPDFObjectHandle trailer = this->m->pdf.getTrailer();
+    if (trailer.hasKey("/ID"))
+    {
+        return trailer.getKey("/ID").getArrayItem(0).getStringValue();
+    }
+    else
+    {
+        return "";
+    }
+}
+
 void
 QPDFWriter::generateID()
 {
@@ -2042,12 +2084,9 @@ QPDFWriter::generateID()
     // generated ID for both.
 
     this->m->id2 = result;
-    if (trailer.hasKey("/ID"))
-    {
-	// Note: keep /ID from old file even if --static-id was given.
-	this->m->id1 = trailer.getKey("/ID").getArrayItem(0).getStringValue();
-    }
-    else
+    // Note: keep /ID from old file even if --static-id was given.
+    this->m->id1 = getOriginalID1();
+    if (this->m->id1.empty())
     {
 	this->m->id1 = this->m->id2;
     }
@@ -2926,11 +2965,24 @@ QPDFWriter::writeLinearized()
 
     // Write file in two passes.  Part numbers refer to PDF spec 1.4.
 
+    FILE* lin_pass1_file = 0;
     for (int pass = 1; pass <= 2; ++pass)
     {
 	if (pass == 1)
 	{
-	    pushDiscardFilter();
+            if (! this->m->lin_pass1_filename.empty())
+            {
+                lin_pass1_file =
+                    QUtil::safe_fopen(
+                        this->m->lin_pass1_filename.c_str(), "wb");
+                pushPipeline(
+                    new Pl_StdioFile("linearization pass1", lin_pass1_file));
+                activatePipelineStack();
+            }
+            else
+            {
+                pushDiscardFilter();
+            }
             if (this->m->deterministic_id)
             {
                 pushMD5Pipeline();
@@ -3170,6 +3222,20 @@ QPDFWriter::writeLinearized()
 
 	    // Restore hint offset
 	    this->m->xref[hint_id] = QPDFXRefEntry(1, hint_offset, 0);
+            if (lin_pass1_file)
+            {
+                // Write some debugging information
+                fprintf(lin_pass1_file, "%% hint_offset=%s\n",
+                        QUtil::int_to_string(hint_offset).c_str());
+                fprintf(lin_pass1_file, "%% hint_length=%s\n",
+                        QUtil::int_to_string(hint_length).c_str());
+                fprintf(lin_pass1_file, "%% second_xref_offset=%s\n",
+                        QUtil::int_to_string(second_xref_offset).c_str());
+                fprintf(lin_pass1_file, "%% second_xref_end=%s\n",
+                        QUtil::int_to_string(second_xref_end).c_str());
+                fclose(lin_pass1_file);
+                lin_pass1_file = 0;
+            }
 	}
     }
 }
