@@ -4,6 +4,7 @@
 #include <qpdf/Pipeline.hh>
 #include <qpdf/Pl_Flate.hh>
 #include <qpdf/Pl_PNGFilter.hh>
+#include <qpdf/Pl_TIFFPredictor.hh>
 #include <qpdf/Pl_RC4.hh>
 #include <qpdf/Pl_Buffer.hh>
 #include <qpdf/Pl_ASCII85Decoder.hh>
@@ -12,7 +13,7 @@
 #include <qpdf/Pl_RunLength.hh>
 #include <qpdf/Pl_DCT.hh>
 #include <qpdf/Pl_Count.hh>
-
+#include <qpdf/ContentNormalizer.hh>
 #include <qpdf/QTC.hh>
 #include <qpdf/QPDF.hh>
 #include <qpdf/QPDFExc.hh>
@@ -38,6 +39,7 @@ QPDF_Stream::QPDF_Stream(QPDF* qpdf, int objid, int generation,
 	    "stream object instantiated with non-dictionary "
 	    "object for dictionary");
     }
+    setStreamDescription();
 }
 
 QPDF_Stream::~QPDF_Stream()
@@ -84,19 +86,56 @@ QPDF_Stream::getTypeName() const
     return "stream";
 }
 
+void
+QPDF_Stream::setDescription(QPDF* qpdf, std::string const& description)
+{
+    this->QPDFObject::setDescription(qpdf, description);
+    setDictDescription();
+}
+
+void
+QPDF_Stream::setStreamDescription()
+{
+    setDescription(
+        this->qpdf,
+        "stream object " + QUtil::int_to_string(this->objid) + " " +
+        QUtil::int_to_string(this->generation));
+}
+
+void
+QPDF_Stream::setDictDescription()
+{
+    QPDF* qpdf = 0;
+    std::string description;
+    if ((! this->stream_dict.hasObjectDescription()) &&
+        getDescription(qpdf, description))
+    {
+        this->stream_dict.setObjectDescription(
+            qpdf, description + " -> stream dictionary");
+    }
+}
+
 QPDFObjectHandle
 QPDF_Stream::getDict() const
 {
     return this->stream_dict;
 }
 
+bool
+QPDF_Stream::isDataModified() const
+{
+    return (! this->token_filters.empty());
+}
+
 PointerHolder<Buffer>
 QPDF_Stream::getStreamData(qpdf_stream_decode_level_e decode_level)
 {
     Pl_Buffer buf("stream data buffer");
     if (! pipeStreamData(&buf, 0, decode_level, false, false))
     {
-	throw std::logic_error("getStreamData called on unfilterable stream");
+	throw QPDFExc(qpdf_e_unsupported, qpdf->getFilename(),
+                      "", this->offset,
+                      "getStreamData called on unfilterable stream");
     }
     QTC::TC("qpdf", "QPDF_Stream getStreamData");
     return buf.getBuffer();
@@ -114,7 +153,9 @@ QPDF_Stream::getRawStreamData()
 bool
 QPDF_Stream::understandDecodeParams(
     std::string const& filter, QPDFObjectHandle decode_obj,
-    int& predictor, int& columns, bool& early_code_change)
+    int& predictor, int& columns,
+    int& colors, int& bits_per_component,
+    bool& early_code_change)
 {
     bool filterable = true;
     std::set<std::string> keys = decode_obj.getKeys();
@@ -122,13 +163,15 @@ QPDF_Stream::understandDecodeParams(
          iter != keys.end(); ++iter)
     {
         std::string const& key = *iter;
-        if ((filter == "/FlateDecode") && (key == "/Predictor"))
+        if (((filter == "/FlateDecode") || (filter == "/LZWDecode")) &&
+            (key == "/Predictor"))
         {
             QPDFObjectHandle predictor_obj = decode_obj.getKey(key);
             if (predictor_obj.isInteger())
             {
                 predictor = predictor_obj.getIntValue();
-                if (! ((predictor == 1) || (predictor == 12)))
+                if (! ((predictor == 1) || (predictor == 2) ||
+                       ((predictor >= 10) && (predictor <= 15))))
                 {
                     filterable = false;
                 }
@@ -155,12 +198,26 @@ QPDF_Stream::understandDecodeParams(
                 filterable = false;
             }
         }
-        else if (key == "/Columns")
+        else if ((key == "/Columns") ||
+                 (key == "/Colors") ||
+                 (key == "/BitsPerComponent"))
         {
-            QPDFObjectHandle columns_obj = decode_obj.getKey(key);
-            if (columns_obj.isInteger())
+            QPDFObjectHandle param_obj = decode_obj.getKey(key);
+            if (param_obj.isInteger())
             {
-                columns = columns_obj.getIntValue();
+                int val = param_obj.getIntValue();
+                if (key == "/Columns")
+                {
+                    columns = val;
+                }
+                else if (key == "/Colors")
+                {
+                    colors = val;
+                }
+                else if (key == "/BitsPerComponent")
+                {
+                    bits_per_component = val;
+                }
             }
             else
             {
@@ -190,6 +247,7 @@ QPDF_Stream::filterable(std::vector<std::string>& filters,
                         bool& specialized_compression,
                         bool& lossy_compression,
 			int& predictor, int& columns,
+                        int& colors, int& bits_per_component,
 			bool& early_code_change)
 {
     if (filter_abbreviations.empty())
@@ -295,6 +353,8 @@ QPDF_Stream::filterable(std::vector<std::string>& filters,
     // Initialize values to their defaults as per the PDF spec
     predictor = 1;
     columns = 0;
+    colors = 1;
+    bits_per_component = 8;
     early_code_change = true;
 
     // See if we can support any decode parameters that are specified.
@@ -344,7 +404,8 @@ QPDF_Stream::filterable(std::vector<std::string>& filters,
         {
             if (! understandDecodeParams(
                     filters.at(i), decode_item,
-                    predictor, columns, early_code_change))
+                    predictor, columns, colors, bits_per_component,
+                    early_code_change))
             {
                 filterable = false;
             }
@@ -378,6 +439,8 @@ QPDF_Stream::pipeStreamData(Pipeline* pipeline,
     std::vector<std::string> filters;
     int predictor = 1;
     int columns = 0;
+    int colors = 1;
+    int bits_per_component = 8;
     bool early_code_change = true;
     bool specialized_compression = false;
     bool lossy_compression = false;
@@ -385,7 +448,9 @@ QPDF_Stream::pipeStreamData(Pipeline* pipeline,
     if (filter)
     {
 	filter = filterable(filters, specialized_compression, lossy_compression,
-                            predictor, columns, early_code_change);
+                            predictor, columns,
+                            colors, bits_per_component,
+                            early_code_change);
         if ((decode_level < qpdf_dl_all) && lossy_compression)
         {
             filter = false;
@@ -411,40 +476,65 @@ QPDF_Stream::pipeStreamData(Pipeline* pipeline,
     // create to be deleted when this function finishes.
     std::vector<PointerHolder<Pipeline> > to_delete;
 
+    PointerHolder<ContentNormalizer> normalizer;
     if (filter)
     {
 	if (encode_flags & qpdf_ef_compress)
 	{
-	    pipeline = new Pl_Flate("compress object stream", pipeline,
+	    pipeline = new Pl_Flate("compress stream", pipeline,
 				    Pl_Flate::a_deflate);
 	    to_delete.push_back(pipeline);
 	}
 
 	if (encode_flags & qpdf_ef_normalize)
 	{
-	    pipeline = new Pl_QPDFTokenizer("normalizer", pipeline);
+            normalizer = new ContentNormalizer();
+	    pipeline = new Pl_QPDFTokenizer(
+                "normalizer", normalizer.getPointer(), pipeline);
 	    to_delete.push_back(pipeline);
 	}
 
+        for (std::vector<PointerHolder<
+                 QPDFObjectHandle::TokenFilter> >::reverse_iterator iter =
+                 this->token_filters.rbegin();
+             iter != this->token_filters.rend(); ++iter)
+        {
+            pipeline = new Pl_QPDFTokenizer(
+                "token filter", (*iter).getPointer(), pipeline);
+            to_delete.push_back(pipeline);
+        }
+
 	for (std::vector<std::string>::reverse_iterator iter = filters.rbegin();
 	     iter != filters.rend(); ++iter)
 	{
 	    std::string const& filter = *iter;
+
+            if ((filter == "/FlateDecode") || (filter == "/LZWDecode"))
+            {
+                if ((predictor >= 10) && (predictor <= 15))
+                {
+                    QTC::TC("qpdf", "QPDF_Stream PNG filter");
+                    pipeline = new Pl_PNGFilter(
+                        "png decode", pipeline, Pl_PNGFilter::a_decode,
+                        columns, colors, bits_per_component);
+                    to_delete.push_back(pipeline);
+                }
+                else if (predictor == 2)
+                {
+                    QTC::TC("qpdf", "QPDF_Stream TIFF predictor");
+                    pipeline = new Pl_TIFFPredictor(
+                        "tiff decode", pipeline, Pl_TIFFPredictor::a_decode,
+                        columns, colors, bits_per_component);
+                    to_delete.push_back(pipeline);
+                }
+            }
+
 	    if (filter == "/Crypt")
 	    {
 		// Ignore -- handled by pipeStreamData
 	    }
 	    else if (filter == "/FlateDecode")
 	    {
-		if (predictor == 12)
-		{
-		    QTC::TC("qpdf", "QPDF_Stream PNG filter");
-		    pipeline = new Pl_PNGFilter(
-			"png decode", pipeline, Pl_PNGFilter::a_decode,
-			columns, 0 /* not used */);
-		    to_delete.push_back(pipeline);
-		}
-
 		pipeline = new Pl_Flate("stream inflate",
 					pipeline, Pl_Flate::a_inflate);
 		to_delete.push_back(pipeline);
@@ -547,6 +637,33 @@ QPDF_Stream::pipeStreamData(Pipeline* pipeline,
         }
     }
 
+    if (filter &&
+        (! suppress_warnings) &&
+        normalizer.getPointer() &&
+        normalizer->anyBadTokens())
+    {
+        warn(QPDFExc(qpdf_e_damaged_pdf, qpdf->getFilename(),
+                     "", this->offset,
+                     "content normalization encountered bad tokens"));
+        if (normalizer->lastTokenWasBad())
+        {
+            QTC::TC("qpdf", "QPDF_Stream bad token at end during normalize");
+            warn(QPDFExc(qpdf_e_damaged_pdf, qpdf->getFilename(),
+                         "", this->offset,
+                         "normalized content ended with a bad token;"
+                         " you may be able to resolve this by"
+                         " coalescing content streams in combination"
+                         " with normalizing content. From the command"
+                         " line, specify --coalesce-contents"));
+        }
+        warn(QPDFExc(qpdf_e_damaged_pdf, qpdf->getFilename(),
+                     "", this->offset,
+                     "Resulting stream data may be corrupted but is"
+                     " may still useful for manual inspection."
+                     " For more information on this warning, search"
+                     " for content normalization in the manual."));
+    }
+
     return filter;
 }
 
@@ -571,6 +688,13 @@ QPDF_Stream::replaceStreamData(
     replaceFilterData(filter, decode_parms, 0);
 }
 
+void
+QPDF_Stream::addTokenFilter(
+    PointerHolder<QPDFObjectHandle::TokenFilter> token_filter)
+{
+    this->token_filters.push_back(token_filter);
+}
+
 void
 QPDF_Stream::replaceFilterData(QPDFObjectHandle const& filter,
 			       QPDFObjectHandle const& decode_parms,
@@ -594,6 +718,7 @@ void
 QPDF_Stream::replaceDict(QPDFObjectHandle new_dict)
 {
     this->stream_dict = new_dict;
+    setDictDescription();
     QPDFObjectHandle length_obj = new_dict.getKey("/Length");
     if (length_obj.isInteger())
     {
