@@ -8,6 +8,43 @@
 // be used.
 #include <string.h>
 
+// First element is 128
+static unsigned short pdf_doc_to_unicode[] = {
+    0x2022,    // 0x80    BULLET
+    0x2020,    // 0x81    DAGGER
+    0x2021,    // 0x82    DOUBLE DAGGER
+    0x2026,    // 0x83    HORIZONTAL ELLIPSIS
+    0x2014,    // 0x84    EM DASH
+    0x2013,    // 0x85    EN DASH
+    0x0192,    // 0x86    SMALL LETTER F WITH HOOK
+    0x2044,    // 0x87    FRACTION SLASH (solidus)
+    0x2039,    // 0x88    SINGLE LEFT-POINTING ANGLE QUOTATION MARK
+    0x203a,    // 0x89    SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
+    0x2212,    // 0x8a    MINUS SIGN
+    0x2030,    // 0x8b    PER MILLE SIGN
+    0x201e,    // 0x8c    DOUBLE LOW-9 QUOTATION MARK (quotedblbase)
+    0x201c,    // 0x8d    LEFT DOUBLE QUOTATION MARK (double quote left)
+    0x201d,    // 0x8e    RIGHT DOUBLE QUOTATION MARK (quotedblright)
+    0x2018,    // 0x8f    LEFT SINGLE QUOTATION MARK (quoteleft)
+    0x2019,    // 0x90    RIGHT SINGLE QUOTATION MARK (quoteright)
+    0x201a,    // 0x91    SINGLE LOW-9 QUOTATION MARK (quotesinglbase)
+    0x2122,    // 0x92    TRADE MARK SIGN
+    0xfb01,    // 0x93    LATIN SMALL LIGATURE FI
+    0xfb02,    // 0x94    LATIN SMALL LIGATURE FL
+    0x0141,    // 0x95    LATIN CAPITAL LETTER L WITH STROKE
+    0x0152,    // 0x96    LATIN CAPITAL LIGATURE OE
+    0x0160,    // 0x97    LATIN CAPITAL LETTER S WITH CARON
+    0x0178,    // 0x98    LATIN CAPITAL LETTER Y WITH DIAERESIS
+    0x017d,    // 0x99    LATIN CAPITAL LETTER Z WITH CARON
+    0x0131,    // 0x9a    LATIN SMALL LETTER DOTLESS I
+    0x0142,    // 0x9b    LATIN SMALL LETTER L WITH STROKE
+    0x0153,    // 0x9c    LATIN SMALL LIGATURE OE
+    0x0161,    // 0x9d    LATIN SMALL LETTER S WITH CARON
+    0x017e,    // 0x9e    LATIN SMALL LETTER Z WITH CARON
+    0xfffd,    // 0x9f    UNDEFINED
+    0x20ac,    // 0xa0    EURO SIGN
+};
+
 // See above about ctype.
 static bool is_ascii_printable(unsigned char ch)
 {
@@ -55,7 +92,7 @@ QPDF_String::unparse(bool force_binary)
 	int consecutive_printable = 0;
 	for (unsigned int i = 0; i < this->val.length(); ++i)
 	{
-	    char ch = this->val[i];
+	    char ch = this->val.at(i);
 	    // Note: do not use locale to determine printability.  The
 	    // PDF specification accepts arbitrary binary data.  Some
 	    // locales imply multibyte characters.  We'll consider
@@ -97,7 +134,7 @@ QPDF_String::unparse(bool force_binary)
 	result += "(";
 	for (unsigned int i = 0; i < this->val.length(); ++i)
 	{
-	    char ch = this->val[i];
+	    char ch = this->val.at(i);
 	    switch (ch)
 	    {
 	      case '\n':
@@ -135,7 +172,7 @@ QPDF_String::unparse(bool force_binary)
 	      default:
 		if (is_iso_latin1_printable(ch))
 		{
-		    result += this->val[i];
+		    result += this->val.at(i);
 		}
 		else
 		{
@@ -164,7 +201,7 @@ QPDF_String::getUTF8Val() const
     std::string result;
     size_t len = this->val.length();
     if ((len >= 2) && (len % 2 == 0) &&
-	(this->val[0] == '\xfe') && (this->val[1] == '\xff'))
+	(this->val.at(0) == '\xfe') && (this->val.at(1) == '\xff'))
     {
 	// This is a Unicode string using big-endian UTF-16.  This
 	// code uses unsigned long and unsigned short to hold
@@ -181,8 +218,8 @@ QPDF_String::getUTF8Val() const
 	    // discarded, and a low codepoint not preceded by a high
 	    // codepoint will just get its low 10 bits output.
 	    unsigned short bits =
-		(static_cast<unsigned char>(this->val[i]) << 8) +
-		static_cast<unsigned char>(this->val[i+1]);
+		(static_cast<unsigned char>(this->val.at(i)) << 8) +
+		static_cast<unsigned char>(this->val.at(i+1));
 	    if ((bits & 0xFC00) == 0xD800)
 	    {
 		codepoint = 0x10000 + ((bits & 0x3FF) << 10);
@@ -209,7 +246,13 @@ QPDF_String::getUTF8Val() const
     {
 	for (unsigned int i = 0; i < len; ++i)
 	{
-	    result += QUtil::toUTF8(static_cast<unsigned char>(this->val[i]));
+            unsigned char ch = static_cast<unsigned char>(this->val.at(i));
+            unsigned short val = ch;
+            if ((ch >= 128) && (ch <= 160))
+            {
+                val = pdf_doc_to_unicode[ch - 128];
+            }
+	    result += QUtil::toUTF8(val);
 	}
     }
     return result;
