@@ -121,10 +121,10 @@ QPDF::isLinearized()
             ++p;
         }
 
-        QPDFTokenizer::Token t1 = readToken(this->m->file, true);
-        QPDFTokenizer::Token t2 = readToken(this->m->file, true);
-        QPDFTokenizer::Token t3 = readToken(this->m->file, true);
-        QPDFTokenizer::Token t4 = readToken(this->m->file, true);
+        QPDFTokenizer::Token t1 = readToken(this->m->file);
+        QPDFTokenizer::Token t2 = readToken(this->m->file);
+        QPDFTokenizer::Token t3 = readToken(this->m->file);
+        QPDFTokenizer::Token t4 = readToken(this->m->file);
         if ((t1.getType() == QPDFTokenizer::tt_integer) &&
             (t2.getType() == QPDFTokenizer::tt_integer) &&
             (t3 == QPDFTokenizer::Token(QPDFTokenizer::tt_word, "obj")) &&
