@@ -1,12 +1,13 @@
 #include <qpdf/Pl_Concatenate.hh>
 #include <qpdf/Pl_Flate.hh>
 #include <qpdf/Pl_Buffer.hh>
+#include <qpdf/QUtil.hh>
 #include <iostream>
 #include <assert.h>
 
 static void pipeStringAndFinish(Pipeline* p, std::string const& str)
 {
-    p->write((unsigned char*)str.c_str(), str.length());
+    p->write(QUtil::unsigned_char_pointer(str), str.length());
     p->finish();
 }
 
@@ -25,7 +26,8 @@ int main(int argc, char* argv[])
     inflate.write(b1_buf->getBuffer(), b1_buf->getSize());
     inflate.finish();
     PointerHolder<Buffer> b2_buf = b2.getBuffer();
-    std::string result((char const*)b2_buf->getBuffer(), b2_buf->getSize());
+    std::string result(reinterpret_cast<char*>(b2_buf->getBuffer()),
+                       b2_buf->getSize());
     if (result == "-one--two-")
     {
         std::cout << "concatenate test passed" << std::endl;
