@@ -61,6 +61,7 @@ struct Options
         encryption_file(0),
         encryption_file_password(0),
         encrypt(false),
+        password_is_hex_key(false),
         keylen(0),
         r2_print(true),
         r2_modify(true),
@@ -89,12 +90,14 @@ struct Options
         qdf_mode(false),
         preserve_unreferenced_objects(false),
         newline_before_endstream(false),
+        coalesce_contents(false),
         show_npages(false),
         deterministic_id(false),
         static_id(false),
         static_aes_iv(false),
         suppress_original_object_id(false),
         show_encryption(false),
+        show_encryption_key(false),
         check_linearization(false),
         show_linearization(false),
         show_xref(false),
@@ -120,6 +123,7 @@ struct Options
     char const* encryption_file;
     char const* encryption_file_password;
     bool encrypt;
+    bool password_is_hex_key;
     std::string user_password;
     std::string owner_password;
     int keylen;
@@ -150,6 +154,8 @@ struct Options
     bool qdf_mode;
     bool preserve_unreferenced_objects;
     bool newline_before_endstream;
+    std::string linearize_pass1;
+    bool coalesce_contents;
     std::string min_version;
     std::string force_version;
     bool show_npages;
@@ -158,6 +164,7 @@ struct Options
     bool static_aes_iv;
     bool suppress_original_object_id;
     bool show_encryption;
+    bool show_encryption_key;
     bool check_linearization;
     bool show_linearization;
     bool show_xref;
@@ -218,6 +225,9 @@ provided last take precedence.\n\
 Basic Options\n\
 -------------\n\
 \n\
+--version               show version of qpdf\n\
+--copyright             show qpdf's copyright and license information\n\
+--help                  show command-line argument help\n\
 --password=password     specify a password for accessing encrypted files\n\
 --verbose               provide additional informational output\n\
 --linearize             generated a linearized (web optimized) file\n\
@@ -227,6 +237,7 @@ Basic Options\n\
                         parameters are being copied\n\
 --encrypt options --    generate an encrypted file\n\
 --decrypt               remove any encryption on the file\n\
+--password-is-hex-key   treat primary password option as a hex-encoded key\n\
 --pages options --      select specific pages from one or more files\n\
 --rotate=[+|-]angle:page-range\n\
                         rotate each specified page 90, 180, or 270 degrees\n\
@@ -240,6 +251,11 @@ parameters will be copied, including both user and owner passwords, even\n\
 if the user password is used to open the other file.  This works even if\n\
 the owner password is not known.\n\
 \n\
+The --password-is-hex-key option overrides the normal computation of\n\
+encryption keys. It only applies to the password used to open the main\n\
+file. This option is not ordinarily useful but can be helpful for forensic\n\
+or investigatory purposes. See manual for further discussion.\n\
+\n\
 The --rotate flag can be used to specify pages to rotate pages either\n\
 90, 180, or 270 degrees. The page range is specified in the same\n\
 format as with the --pages option, described below. Repeat the option\n\
@@ -342,11 +358,12 @@ input.\n\
 \n\
 The page range is a set of numbers separated by commas, ranges of\n\
 numbers separated dashes, or combinations of those.  The character\n\
-\"z\" represents the last page.  Pages can appear in any order.  Ranges\n\
-can appear with a high number followed by a low number, which causes the\n\
-pages to appear in reverse.  Repeating a number will cause an error, but\n\
-the manual discusses a workaround should you really want to include the\n\
-same page twice.\n\
+\"z\" represents the last page.  A number preceded by an \"r\" indicates\n\
+to count from the end, so \"r3-r1\" would be the last three pages of the\n\
+document.  Pages can appear in any order.  Ranges can appear with a\n\
+high number followed by a low number, which causes the pages to appear in\n\
+reverse.  Repeating a number will cause an error, but the manual discusses\n\
+a workaround should you really want to include the same page twice.\n\
 \n\
 If the page range is omitted, the range of 1-z is assumed.  qpdf decides\n\
 that the page range is omitted if the range argument is either -- or a\n\
@@ -355,6 +372,17 @@ valid file name and not a valid range.\n\
 See the manual for examples and a discussion of additional subtleties.\n\
 \n\
 \n\
+Advanced Parsing Options\n\
+-------------------------------\n\
+\n\
+These options control aspects of how qpdf reads PDF files. Mostly these are\n\
+of use to people who are working with damaged files. There is little reason\n\
+to use these options unless you are trying to solve specific problems.\n\
+\n\
+--suppress-recovery       prevents qpdf from attempting to recover damaged files\n\
+--ignore-xref-streams     tells qpdf to ignore any cross-reference streams\n\
+\n\
+\n\
 Advanced Transformation Options\n\
 -------------------------------\n\
 \n\
@@ -366,12 +394,13 @@ familiar with the PDF file format or who are PDF developers.\n\
 --compress-streams=[yn]   controls whether to compress streams on output\n\
 --decode-level=option     controls how to filter streams from the input\n\
 --normalize-content=[yn]  enables or disables normalization of content streams\n\
---suppress-recovery       prevents qpdf from attempting to recover damaged files\n\
 --object-streams=mode     controls handing of object streams\n\
---ignore-xref-streams     tells qpdf to ignore any cross-reference streams\n\
 --preserve-unreferenced   preserve unreferenced objects\n\
 --newline-before-endstream  always put a newline before endstream\n\
+--coalesce-contents       force all pages' content to be a single stream\n\
 --qdf                     turns on \"QDF mode\" (below)\n\
+--linearize-pass1=file    write intermediate pass of linearized file\n\
+                          for debugging\n\
 --min-version=version     sets the minimum PDF version of the output file\n\
 --force-version=version   forces this to be the PDF version of the output file\n\
 \n\
@@ -425,6 +454,7 @@ automated test suites for software that uses the qpdf library.\n\
                           This is option is not secure!  FOR TESTING ONLY!\n\
 --no-original-object-ids  suppress original object ID comments in qdf mode\n\
 --show-encryption         quickly show encryption parameters\n\
+--show-encryption-key     when showing encryption, reveal the actual key\n\
 --check-linearization     check file integrity and linearization status\n\
 --show-linearization      check and show all linearization data\n\
 --show-xref               show the contents of the cross-reference table\n\
@@ -492,7 +522,7 @@ static std::string show_encryption_method(QPDF::encryption_method_e method)
     return result;
 }
 
-static void show_encryption(QPDF& pdf)
+static void show_encryption(QPDF& pdf, Options& o)
 {
     // Extract /P from /Encrypt
     int R = 0;
@@ -511,8 +541,14 @@ static void show_encryption(QPDF& pdf)
 	std::cout << "R = " << R << std::endl;
 	std::cout << "P = " << P << std::endl;
 	std::string user_password = pdf.getTrimmedUserPassword();
-	std::cout << "User password = " << user_password << std::endl
-                  << "extract for accessibility: "
+        std::string encryption_key = pdf.getEncryptionKey();
+	std::cout << "User password = " << user_password << std::endl;
+        if (o.show_encryption_key)
+        {
+            std::cout << "Encryption key = "
+                      << QUtil::hex_encode(encryption_key) << std::endl;
+        }
+        std::cout << "extract for accessibility: "
 		  << show_bool(pdf.allowAccessibility()) << std::endl
                   << "extract for any purpose: "
 		  << show_bool(pdf.allowExtractAll()) << std::endl
@@ -542,6 +578,22 @@ static void show_encryption(QPDF& pdf)
     }
 }
 
+static int maybe_from_end(int num, bool from_end, int max)
+{
+    if (from_end)
+    {
+        if (num > max)
+        {
+            num = 0;
+        }
+        else
+        {
+            num = max + 1 - num;
+        }
+    }
+    return num;
+}
+
 static std::vector<int> parse_numrange(char const* range, int max,
                                        bool throw_error = false)
 {
@@ -558,6 +610,7 @@ static std::vector<int> parse_numrange(char const* range, int max,
                st_after_number } state = st_top;
         bool last_separator_was_dash = false;
         int cur_number = 0;
+        bool from_end = false;
         while (*p)
         {
             char ch = *p;
@@ -581,14 +634,25 @@ static std::vector<int> parse_numrange(char const* range, int max,
                 state = st_after_number;
                 cur_number = max;
             }
+            else if (ch == 'r')
+            {
+                if (! (state == st_top))
+                {
+                    throw std::runtime_error("r not expected");
+                }
+                state = st_in_number;
+                from_end = true;
+            }
             else if ((ch == ',') || (ch == '-'))
             {
                 if (! ((state == st_in_number) || (state == st_after_number)))
                 {
                     throw std::runtime_error("unexpected separator");
                 }
+                cur_number = maybe_from_end(cur_number, from_end, max);
                 work.push_back(cur_number);
                 cur_number = 0;
+                from_end = false;
                 if (ch == ',')
                 {
                     state = st_top;
@@ -614,6 +678,7 @@ static std::vector<int> parse_numrange(char const* range, int max,
         }
         if ((state == st_in_number) || (state == st_after_number))
         {
+            cur_number = maybe_from_end(cur_number, from_end, max);
             work.push_back(cur_number);
         }
         else
@@ -1156,22 +1221,29 @@ static void read_args_from_file(char const* filename,
 
 static void handle_help_version(int argc, char* argv[])
 {
+    // Make sure the output looks right on an 80-column display.
+
     if ((argc == 2) &&
         ((strcmp(argv[1], "--version") == 0) ||
          (strcmp(argv[1], "-version") == 0)))
     {
-        // make_dist looks for the line of code here that actually
-        // prints the version number, so read make_dist if you change
-        // anything other than the version number.  Don't worry about
-        // the numbers.  That's just a guide to 80 columns so that the
-        // help message looks right on an 80-column display.
+        std::cout
+            << whoami << " version " << QPDF::QPDFVersion() << std::endl
+            << "Run " << whoami << " --copyright to see copyright and license information."
+            << std::endl;
+        exit(0);
+    }
 
+    if ((argc == 2) &&
+             ((strcmp(argv[1], "--copyright") == 0) ||
+              (strcmp(argv[1], "-copyright") == 0)))
+    {
         //               1         2         3         4         5         6         7         8
         //      12345678901234567890123456789012345678901234567890123456789012345678901234567890
         std::cout
             << whoami << " version " << QPDF::QPDFVersion() << std::endl
             << std::endl
-            << "Copyright (c) 2005-2017 Jay Berkenbilt"
+            << "Copyright (c) 2005-2018 Jay Berkenbilt"
             << std::endl
             << "QPDF is licensed under the Apache License, Version 2.0 (the \"License\");"
             << std::endl
@@ -1330,6 +1402,10 @@ static void parse_options(int argc, char* argv[], Options& o)
                 o.encrypt = false;
                 o.copy_encryption = false;
             }
+            else if (strcmp(arg, "password-is-hex-key") == 0)
+            {
+                o.password_is_hex_key = true;
+            }
             else if (strcmp(arg, "copy-encryption") == 0)
             {
                 if (parameter == 0)
@@ -1501,6 +1577,19 @@ static void parse_options(int argc, char* argv[], Options& o)
             {
                 o.newline_before_endstream = true;
             }
+            else if (strcmp(arg, "linearize-pass1") == 0)
+            {
+                if (parameter == 0)
+                {
+                    usage("--linearize-pass1 be given as"
+                          "--linearize-pass1=filename");
+                }
+                o.linearize_pass1 = parameter;
+            }
+            else if (strcmp(arg, "coalesce-contents") == 0)
+            {
+                o.coalesce_contents = true;
+            }
             else if (strcmp(arg, "min-version") == 0)
             {
                 if (parameter == 0)
@@ -1550,6 +1639,10 @@ static void parse_options(int argc, char* argv[], Options& o)
                 o.show_encryption = true;
                 o.require_outfile = false;
             }
+            else if (strcmp(arg, "show-encryption-key") == 0)
+            {
+                o.show_encryption_key = true;
+            }
             else if (strcmp(arg, "check-linearization") == 0)
             {
                 o.check_linearization = true;
@@ -1664,6 +1757,10 @@ static void set_qpdf_options(QPDF& pdf, Options& o)
     {
         pdf.setAttemptRecovery(false);
     }
+    if (o.password_is_hex_key)
+    {
+        pdf.setPasswordIsHexKey(true);
+    }
 }
 
 static void do_check(QPDF& pdf, Options& o, int& exit_code)
@@ -1684,7 +1781,7 @@ static void do_check(QPDF& pdf, Options& o, int& exit_code)
                       << pdf.getExtensionLevel();
         }
         std::cout << std::endl;
-        show_encryption(pdf);
+        show_encryption(pdf, o);
         if (pdf.isLinearized())
         {
             std::cout << "File is linearized\n";
@@ -1719,9 +1816,7 @@ static void do_check(QPDF& pdf, Options& o, int& exit_code)
             ++pageno;
             try
             {
-                QPDFObjectHandle::parseContentStream(
-                    (*iter).getKey("/Contents"),
-                    &discard_contents);
+                (*iter).parsePageContents(&discard_contents);
             }
             catch (QPDFExc& e)
             {
@@ -1868,7 +1963,7 @@ static void do_inspection(QPDF& pdf, Options& o)
     }
     if (o.show_encryption)
     {
-        show_encryption(pdf);
+        show_encryption(pdf, o);
     }
     if (o.check_linearization)
     {
@@ -1912,6 +2007,19 @@ static void do_inspection(QPDF& pdf, Options& o)
     }
 }
 
+static void handle_transformations(QPDF& pdf, Options& o)
+{
+    if (o.coalesce_contents)
+    {
+        std::vector<QPDFObjectHandle> pages = pdf.getAllPages();
+        for (std::vector<QPDFObjectHandle>::iterator iter = pages.begin();
+             iter != pages.end(); ++iter)
+        {
+            (*iter).coalesceContentStreams();
+        }
+    }
+}
+
 static void handle_page_specs(QPDF& pdf, Options& o,
                               std::vector<PointerHolder<QPDF> >& page_heap)
 {
@@ -2176,6 +2284,10 @@ static void set_writer_options(QPDF& pdf, Options& o, QPDFWriter& w)
     {
         w.setLinearization(true);
     }
+    if (! o.linearize_pass1.empty())
+    {
+        w.setLinearizationPass1Filename(o.linearize_pass1);
+    }
     if (o.object_stream_set)
     {
         w.setObjectStreamMode(o.object_stream_mode);
@@ -2330,6 +2442,7 @@ int main(int argc, char* argv[])
             pdf.processFile(o.infilename, o.password);
         }
 
+        handle_transformations(pdf, o);
         std::vector<PointerHolder<QPDF> > page_heap;
         if (! o.page_specs.empty())
         {
