@@ -97,6 +97,35 @@ ParserCallbacks::handleEOF()
     std::cout << "-EOF-" << std::endl;
 }
 
+class TokenFilter: public QPDFObjectHandle::TokenFilter
+{
+  public:
+    TokenFilter()
+    {
+    }
+    virtual ~TokenFilter()
+    {
+    }
+    virtual void handleToken(QPDFTokenizer::Token const& t)
+    {
+        if (t == QPDFTokenizer::Token(QPDFTokenizer::tt_string, "Potato"))
+        {
+            // Exercise unparsing of strings by token constructor
+            writeToken(
+                QPDFTokenizer::Token(QPDFTokenizer::tt_string, "Salad"));
+        }
+        else
+        {
+            writeToken(t);
+        }
+    }
+    virtual void handleEOF()
+    {
+        writeToken(QPDFTokenizer::Token(QPDFTokenizer::tt_name, "/bye"));
+        write("\n");
+    }
+};
+
 static std::string getPageContents(QPDFObjectHandle page)
 {
     PointerHolder<Buffer> b1 =
@@ -314,15 +343,15 @@ void runtest(int n, char const* filename1, char const* arg2)
 	    std::cout.flush();
 	    QUtil::binary_stdout();
 	    PointerHolder<Pl_StdioFile> out = new Pl_StdioFile("raw", stdout);
-	    qtest.pipeStreamData(out.getPointer(), false, false, false);
+	    qtest.pipeStreamData(out.getPointer(), 0, qpdf_dl_none);
 
 	    std::cout << std::endl << "Uncompressed stream data:" << std::endl;
-	    if (qtest.pipeStreamData(0, true, false, false))
+	    if (qtest.pipeStreamData(0, 0, qpdf_dl_all))
 	    {
 		std::cout.flush();
 		QUtil::binary_stdout();
 		out = new Pl_StdioFile("filtered", stdout);
-		qtest.pipeStreamData(out.getPointer(), true, false, false);
+		qtest.pipeStreamData(out.getPointer(), 0, qpdf_dl_all);
 		std::cout << std::endl << "End of stream data" << std::endl;
 	    }
 	    else
@@ -362,7 +391,7 @@ void runtest(int n, char const* filename1, char const* arg2)
 	QPDFObjectHandle contents = page.getKey("/Contents");
 	QUtil::binary_stdout();
 	PointerHolder<Pl_StdioFile> out = new Pl_StdioFile("filtered", stdout);
-	contents.pipeStreamData(out.getPointer(), true, false, false);
+	contents.pipeStreamData(out.getPointer(), 0, qpdf_dl_generalized);
     }
     else if (n == 3)
     {
@@ -375,7 +404,8 @@ void runtest(int n, char const* filename1, char const* arg2)
 	    QUtil::binary_stdout();
 	    PointerHolder<Pl_StdioFile> out =
 		new Pl_StdioFile("tokenized stream", stdout);
-	    stream.pipeStreamData(out.getPointer(), true, true, false);
+	    stream.pipeStreamData(out.getPointer(),
+                                  qpdf_ef_normalize, qpdf_dl_generalized);
 	}
     }
     else if (n == 4)
@@ -497,7 +527,7 @@ void runtest(int n, char const* filename1, char const* arg2)
 	    throw std::logic_error("test 6 run on file with no metadata");
 	}
 	Pl_Buffer bufpl("buffer");
-	metadata.pipeStreamData(&bufpl, false, false, false);
+	metadata.pipeStreamData(&bufpl, 0, qpdf_dl_none);
 	Buffer* buf = bufpl.getBuffer();
 	unsigned char const* data = buf->getBuffer();
 	bool cleartext = false;
@@ -567,7 +597,7 @@ void runtest(int n, char const* filename1, char const* arg2)
 	    qstream.getStreamData();
 	    std::cout << "oops -- getStreamData didn't throw" << std::endl;
 	}
-	catch (std::logic_error const& e)
+        catch (std::exception const& e)
 	{
 	    std::cout << "exception: " << e.what() << std::endl;
 	}
@@ -1277,7 +1307,7 @@ void runtest(int n, char const* filename1, char const* arg2)
                 QPDFObjectHandle stream = item.getKey("/EF").getKey("/F");
                 Pl_Buffer p1("buffer");
                 Pl_Flate p2("compress", &p1, Pl_Flate::a_inflate);
-                stream.pipeStreamData(&p2, false, false, false);
+                stream.pipeStreamData(&p2, 0, qpdf_dl_none);
                 PointerHolder<Buffer> buf = p1.getBuffer();
                 std::string data = std::string(
                     reinterpret_cast<char const*>(buf->getBuffer()),
@@ -1295,9 +1325,8 @@ void runtest(int n, char const* filename1, char const* arg2)
              iter != pages.end(); ++iter)
         {
             QPDFObjectHandle page = *iter;
-            QPDFObjectHandle contents = page.getKey("/Contents");
             ParserCallbacks cb;
-            QPDFObjectHandle::parseContentStream(contents, &cb);
+            page.parsePageContents(&cb);
         }
     }
     else if (n == 38)
@@ -1309,6 +1338,118 @@ void runtest(int n, char const* filename1, char const* arg2)
             std::cout << qtest.getArrayItem(i).unparseResolved() << std::endl;
         }
     }
+    else if (n == 39)
+    {
+        // Display image filter and color set for each image on each page
+        std::vector<QPDFObjectHandle> pages = pdf.getAllPages();
+        int pageno = 0;
+        for (std::vector<QPDFObjectHandle>::iterator p_iter =
+                 pages.begin();
+             p_iter != pages.end(); ++p_iter)
+        {
+            std::cout << "page " << ++pageno << std::endl;
+	    std::map<std::string, QPDFObjectHandle> images =
+		(*p_iter).getPageImages();
+	    for (std::map<std::string, QPDFObjectHandle>::iterator i_iter =
+		     images.begin(); i_iter != images.end(); ++i_iter)
+	    {
+                QPDFObjectHandle image_dict = (*i_iter).second.getDict();
+                std::cout << "filter: "
+                          << image_dict.getKey("/Filter").unparseResolved()
+                          << ", color space: "
+                          << image_dict.getKey("/ColorSpace").unparseResolved()
+                          << std::endl;
+            }
+        }
+    }
+    else if (n == 40)
+    {
+        // Write PCLm. This requires specially crafted PDF files. This
+        // feature was implemented by Sahil Arora
+        // <sahilarora.535@gmail.com> as part of a Google Summer of
+        // Code project in 2017.
+        assert(arg2 != 0);
+        QPDFWriter w(pdf, arg2);
+        w.setPCLm(true);
+        w.setStaticID(true);
+        w.write();
+    }
+    else if (n == 41)
+    {
+        // Apply a token filter. This test case is crafted to work
+        // with coalesce.pdf.
+        std::vector<QPDFObjectHandle> pages = pdf.getAllPages();
+        for (std::vector<QPDFObjectHandle>::iterator iter =
+                 pages.begin();
+             iter != pages.end(); ++iter)
+        {
+            (*iter).addContentTokenFilter(new TokenFilter);
+        }
+        QPDFWriter w(pdf, "a.pdf");
+	w.setQDFMode(true);
+        w.setStaticID(true);
+        w.write();
+    }
+    else if (n == 42)
+    {
+        // Access objects as wrong type. This test case is crafted to
+        // work with object-types.pdf.
+        QPDFObjectHandle qtest = pdf.getTrailer().getKey("/QTest");
+        QPDFObjectHandle array = qtest.getKey("/Dictionary").getKey("/Key2");
+        QPDFObjectHandle dictionary = qtest.getKey("/Dictionary");
+        QPDFObjectHandle integer = qtest.getKey("/Integer");
+        QPDFObjectHandle null = QPDFObjectHandle::newNull();
+        assert(array.isArray());
+        assert(dictionary.isDictionary());
+        assert("" == qtest.getStringValue());
+        array.getArrayItem(-1).assertNull();
+        array.getArrayItem(16059).assertNull();
+        integer.getArrayItem(0).assertNull();
+        integer.appendItem(null);
+        array.eraseItem(-1);
+        array.eraseItem(16059);
+        integer.eraseItem(0);
+        integer.insertItem(0, null);
+        integer.setArrayFromVector(std::vector<QPDFObjectHandle>());
+        integer.setArrayItem(0, null);
+        assert(0 == integer.getArrayNItems());
+        assert(integer.getArrayAsVector().empty());
+        assert(false == integer.getBoolValue());
+        assert(integer.getDictAsMap().empty());
+        assert(integer.getKeys().empty());
+        assert(false == integer.hasKey("/Potato"));
+        integer.removeKey("/Potato");
+        integer.replaceOrRemoveKey("/Potato", null);
+        integer.replaceOrRemoveKey("/Potato", QPDFObjectHandle::newInteger(1));
+        integer.replaceKey("/Potato", QPDFObjectHandle::newInteger(1));
+        qtest.getKey("/Integer").getKey("/Potato");
+        assert(integer.getInlineImageValue().empty());
+        assert(0 == dictionary.getIntValue());
+        assert("/QPDFFakeName" == integer.getName());
+        assert("QPDFFAKE" == integer.getOperatorValue());
+        assert("0.0" == dictionary.getRealValue());
+        assert(integer.getStringValue().empty());
+        assert(integer.getUTF8Value().empty());
+        assert(0.0 == dictionary.getNumericValue());
+        // Make sure error messages are okay for nested values
+        std::cerr << "One error\n";
+        assert(array.getArrayItem(0).getStringValue().empty());
+        std::cerr << "One error\n";
+        assert(dictionary.getKey("/Quack").getStringValue().empty());
+        assert(array.getArrayItem(1).isDictionary());
+        assert(array.getArrayItem(1).getKey("/K").isArray());
+        assert(array.getArrayItem(1).getKey("/K").getArrayItem(0).isName());
+        assert("/V" ==
+               array.getArrayItem(1).getKey("/K").getArrayItem(0).getName());
+        std::cerr << "Two errors\n";
+        assert(array.getArrayItem(16059).getStringValue().empty());
+        std::cerr << "One error\n";
+        array.getArrayItem(1).getKey("/K").getArrayItem(0).getStringValue();
+        // Stream dictionary
+        QPDFObjectHandle page = pdf.getAllPages()[0];
+        assert("/QPDFFakeName" ==
+               page.getKey("/Contents").getDict().getKey("/Potato").getName());
+    }
     else
     {
 	throw std::runtime_error(std::string("invalid test ") +
@@ -1346,7 +1487,7 @@ int main(int argc, char* argv[])
 
     try
     {
-	int n = atoi(argv[1]);
+	int n = QUtil::string_to_int(argv[1]);
 	char const* filename1 = argv[2];
         char const* arg2 = argv[3];
 	runtest(n, filename1, arg2);
