@@ -273,7 +273,7 @@ static void check_image(int pageno, QPDFObjectHandle page)
     QPDFObjectHandle image =
         page.getKey("/Resources").getKey("/XObject").getKey("/Im1");
     ImageChecker ic(pageno);
-    image.pipeStreamData(&ic, true, false, false);
+    image.pipeStreamData(&ic, 0, qpdf_dl_specialized);
 }
 
 static void check_pdf(char const* filename)
@@ -286,8 +286,8 @@ static void check_pdf(char const* filename)
     {
         int pageno = i + 1;
         std::cout << "page " << pageno << " of " << npages << std::endl;
-        check_page_contents(pageno, pages[i]);
-        check_image(pageno, pages[i]);
+        check_page_contents(pageno, pages.at(i));
+        check_image(pageno, pages.at(i));
     }
 }
 
